"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LegacyChainHandler = void 0;
const lisk_codec_1 = require("@liskhq/lisk-codec");
const lisk_db_1 = require("@liskhq/lisk-db");
const schema_1 = require("../consensus/schema");
const storage_1 = require("./storage");
const codec_1 = require("./codec");
const errors_1 = require("./errors");
const validate_1 = require("./validate");
const schemas_1 = require("./schemas");
class LegacyChainHandler {
    constructor(args) {
        this._legacyConfig = args.legacyConfig;
        this._network = args.network;
        this._logger = args.logger;
    }
    async init(args) {
        this._storage = new storage_1.Storage(args.db);
        for (const bracket of this._legacyConfig.brackets) {
            try {
                await this._storage.getLegacyChainBracketInfo(Buffer.from(bracket.snapshotBlockID, 'hex'));
            }
            catch (err) {
                if (!(err instanceof lisk_db_1.NotFoundError)) {
                    throw err;
                }
                await this._storage.setLegacyChainBracketInfo(Buffer.from(bracket.snapshotBlockID), {
                    startHeight: bracket.startHeight,
                    snapshotBlockHeight: bracket.snapshotHeight,
                    lastBlockHeight: bracket.snapshotHeight,
                });
            }
        }
    }
    async sync() {
        for (const bracket of this._legacyConfig.brackets) {
            const encodedBracketInfo = await this._storage.getLegacyChainBracketInfo(Buffer.from(bracket.snapshotBlockID, 'hex'));
            const bracketInfo = lisk_codec_1.codec.decode(schemas_1.legacyChainBracketInfoSchema, encodedBracketInfo);
            if (bracket.startHeight === bracketInfo.lastBlockHeight) {
                continue;
            }
            const lastBlock = (0, codec_1.decodeBlock)(await this._storage.getBlockByHeight(bracketInfo.lastBlockHeight)).block;
            await this._trySyncBlocks(bracket, lastBlock);
        }
        this._network.applyNodeInfo({
            legacy: this._legacyConfig.brackets.map(bracket => Buffer.from(bracket.snapshotBlockID, 'hex')),
        });
        clearTimeout(this._syncTimeout);
    }
    async _trySyncBlocks(bracket, lastBlock) {
        try {
            await this.syncBlocks(bracket, lastBlock);
        }
        catch (err) {
            if (err instanceof errors_1.PeerNotFoundWithLegacyInfo) {
                this._syncTimeout = setTimeout(async () => {
                    await this._trySyncBlocks(bracket, lastBlock);
                }, 120000);
            }
            else {
                throw err;
            }
        }
    }
    async syncBlocks(bracket, legacyBlock) {
        const connectedPeers = this._network.getConnectedPeers();
        const peersWithLegacyInfo = connectedPeers.filter(peer => !!peer.options.legacy.find(snapshotBlockID => snapshotBlockID.equals(Buffer.from(bracket.snapshotBlockID, 'hex'))));
        if (!peersWithLegacyInfo) {
            throw new errors_1.PeerNotFoundWithLegacyInfo('No peer found with legacy info.');
        }
        const randomPeerIndex = Math.trunc(Math.random() * peersWithLegacyInfo.length - 1);
        const { peerId } = peersWithLegacyInfo[randomPeerIndex];
        const p2PRequestPacket = {
            procedure: 'getLegacyBlocksFromId',
            data: legacyBlock.header.id,
        };
        const response = await this._network.requestFromPeer({ ...p2PRequestPacket, peerId });
        const { data } = response;
        let legacyBlocks;
        const applyPenaltyAndRepeat = async (msg) => {
            this._logger.warn({ peerId }, `${msg} Applying a penalty to the peer`);
            this._network.applyPenaltyOnPeer({ peerId, penalty: 100 });
            await this.syncBlocks(bracket, legacyBlock);
        };
        try {
            const { blocks } = lisk_codec_1.codec.decode(schema_1.getBlocksFromIdResponseSchema, data);
            if (blocks.length === 0) {
                await applyPenaltyAndRepeat('Received empty response');
            }
            this._applyValidation(blocks);
            legacyBlocks = blocks.map(block => (0, codec_1.decodeBlock)(block).block);
            if (legacyBlocks.length === 0) {
                await applyPenaltyAndRepeat('received empty blocks');
            }
        }
        catch (err) {
            await applyPenaltyAndRepeat(err.message);
        }
        for (const block of legacyBlocks) {
            if (block.header.height > bracket.startHeight) {
                const payload = block.payload.length ? block.payload : [];
                await this._storage.saveBlock(block.header.id, block.header.height, (0, codec_1.encodeBlock)(block), payload);
            }
        }
        const lastBlock = legacyBlocks[legacyBlocks.length - 1];
        if (lastBlock && lastBlock.header.height > bracket.startHeight) {
            await this._updateBracketInfo(lastBlock, bracket);
            await this.syncBlocks(bracket, lastBlock);
        }
        await this._updateBracketInfo(lastBlock, bracket);
    }
    async _updateBracketInfo(lastBlock, bracket) {
        await this._storage.setLegacyChainBracketInfo(Buffer.from(bracket.snapshotBlockID, 'hex'), {
            startHeight: bracket.startHeight,
            lastBlockHeight: lastBlock === null || lastBlock === void 0 ? void 0 : lastBlock.header.height,
            snapshotBlockHeight: bracket.snapshotHeight,
        });
    }
    _applyValidation(blocks) {
        const sortedBlocks = [];
        for (let i = blocks.length - 1; i >= 0; i -= 1) {
            sortedBlocks.push(blocks[i]);
        }
        const sortedLegacyBlocks = sortedBlocks.map(block => (0, codec_1.decodeBlock)(block).block);
        sortedBlocks.forEach((block, index) => {
            if (index < sortedBlocks.length - 1) {
                (0, validate_1.validateLegacyBlock)(block, sortedLegacyBlocks[index + 1]);
            }
        });
    }
}
exports.LegacyChainHandler = LegacyChainHandler;
//# sourceMappingURL=legacy_chain_handler.js.map