"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LegacyNetworkEndpoint = void 0;
const lisk_codec_1 = require("@liskhq/lisk-codec");
const lisk_validator_1 = require("@liskhq/lisk-validator");
const base_network_endpoint_1 = require("../network/base_network_endpoint");
const constants_1 = require("../consensus/constants");
const schema_1 = require("../consensus/schema");
const storage_1 = require("./storage");
const codec_1 = require("./codec");
const LEGACY_BLOCKS_FROM_IDS_RATE_LIMIT_FREQUENCY = 100;
class LegacyNetworkEndpoint extends base_network_endpoint_1.BaseNetworkEndpoint {
    constructor(args) {
        super(args.network);
        this._logger = args.logger;
        this._network = args.network;
        this._storage = new storage_1.Storage(args.db);
    }
    async handleRPCGetLegacyBlocksFromID(data, peerID) {
        this.addRateLimit(constants_1.NETWORK_LEGACY_GET_BLOCKS_FROM_ID, peerID, LEGACY_BLOCKS_FROM_IDS_RATE_LIMIT_FREQUENCY);
        let rpcBlocksByIdData;
        try {
            rpcBlocksByIdData = lisk_codec_1.codec.decode(schema_1.getBlocksFromIdRequestSchema, data);
        }
        catch (error) {
            this._logger.warn({
                err: error,
                req: data,
                peerID,
            }, `${constants_1.NETWORK_LEGACY_GET_BLOCKS_FROM_ID} response failed on decoding. Applying a penalty to the peer`);
            this._network.applyPenaltyOnPeer({
                peerId: peerID,
                penalty: 100,
            });
            throw error;
        }
        try {
            lisk_validator_1.validator.validate(schema_1.getBlocksFromIdRequestSchema, rpcBlocksByIdData);
        }
        catch (error) {
            this._logger.warn({
                err: error,
                req: data,
                peerID,
            }, `${constants_1.NETWORK_LEGACY_GET_BLOCKS_FROM_ID} response failed on validation. Applying a penalty to the peer`);
            this._network.applyPenaltyOnPeer({
                peerId: peerID,
                penalty: 100,
            });
            throw error;
        }
        const { blockId } = rpcBlocksByIdData;
        let lastBlockHeader;
        try {
            const block = await this._storage.getBlockByID(blockId);
            lastBlockHeader = (0, codec_1.decodeBlock)(block).block.header;
        }
        catch (errors) {
            return lisk_codec_1.codec.encode(schema_1.getBlocksFromIdResponseSchema, { blocks: [] });
        }
        const lastBlockHeight = lastBlockHeader.height;
        const fetchUntilHeight = lastBlockHeight + 100;
        const encodedBlocks = await this._storage.getBlocksByHeightBetween(lastBlockHeight, fetchUntilHeight);
        return lisk_codec_1.codec.encode(schema_1.getBlocksFromIdResponseSchema, { blocks: encodedBlocks });
    }
}
exports.LegacyNetworkEndpoint = LegacyNetworkEndpoint;
//# sourceMappingURL=network_endpoint.js.map